@@ -14,6 +14,7 @@
 ##' @param ziPredictCode zero-inflation code
 ##' @param doPredict flag to enable sds of predictions
 ##' @param whichPredict which observations in model frame represent predictions
+##' @param sparseX see \code{\link{glmmTMB}}
 ##' @keywords internal
 mkTMBStruc <- function(formula, ziformula, dispformula,
                        combForm,
@@ -34,11 +35,21 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
                        whichPredict=integer(0),
                        REML=FALSE,
                        start=NULL,
-                       map=NULL) {
+                       map=NULL,
+                       sparseX=NULL) {
 
   ## handle family specified as naked list
   ## if specified as character or function, should have been converted
   ## to a list with class "family" above ...
+
+  if (is.null(sparseX)) sparseX <- logical(0)
+  for (component in c("cond", "zi", "disp")) {
+      if (!component %in% names(sparseX)) {
+          sparseX[[component]] <- FALSE
+      }
+  }
+  ## make sure the order is right!
+  sparseX <- sparseX[c("cond","zi","disp")]
     
   ## FIXME: (1) should use proper tryCatch below
   if (!is(family,"family")) {
@@ -87,11 +98,11 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
         dispformula[] <- ~0
     }
 
-    condList  <- getXReTrms(formula, mf, fr, contrasts=contrasts)
-    ziList    <- getXReTrms(ziformula, mf, fr, contrasts=contrasts)
+    condList  <- getXReTrms(formula, mf, fr, contrasts=contrasts, sparse=sparseX[["cond"]])
+    ziList    <- getXReTrms(ziformula, mf, fr, contrasts=contrasts, sparse=sparseX[["zi"]])
     dispList  <- getXReTrms(dispformula, mf, fr,
                             ranOK=FALSE, type="dispersion",
-                            contrasts=contrasts)
+                            contrasts=contrasts, sparse=sparseX[["disp"]])
 
     condReStruc <- with(condList, getReStruc(reTrms, ss))
     ziReStruc <- with(ziList, getReStruc(reTrms, ss))
@@ -133,13 +144,22 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
   }
   if (is.null(size)) size <- numeric(0)
 
-    
+
+  denseXval <- function(component,lst) if (sparseX[[component]]) matrix(nrow=0,ncol=0) else lst$X
+  ## need a 'dgTMatrix' (double, general, Triplet representation)
+  sparseXval <- function(component,lst)
+    { if (sparseX[[component]]) lst$X else Matrix::sparseMatrix(dims=c(0,0),i=integer(0),j=integer(0),x=numeric(0),giveCsparse=FALSE) }
+
   data.tmb <- namedList(
-    X = condList$X,
+    X = denseXval("cond",condList),
+    XS = sparseXval("cond",condList),  
     Z = condList$Z,
-    Xzi = ziList$X,
+    Xzi = denseXval("zi",ziList),
+    XziS = sparseXval("zi",ziList),
     Zzi = ziList$Z,
-    Xd = dispList$X,
+    Xd = denseXval("disp",dispList),
+    XdS = sparseXval("disp",dispList),
+    
     ## Zdisp=dispList$Z,
     ## use c() on yobs, size to strip attributes such as 'AsIs'
     ##  (which confuse MakeADFun)
@@ -176,11 +196,11 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
   }
   parameters <- with(data.tmb,
                      list(
-                       beta    = rep(beta_init, ncol(X)),
-                       betazi  = rr0(ncol(Xzi)),
+                       beta    = rep(beta_init, max(ncol(X),ncol(XS))),
+                       betazi  = rr0(max(ncol(Xzi),ncol(XziS))),
                        b       = rep(beta_init, ncol(Z)),
                        bzi     = rr0(ncol(Zzi)),
-                       betad   = rep(betad_init, ncol(Xd)),
+                       betad   = rep(betad_init, max(ncol(Xd),ncol(XdS))),
                        theta   = rr0(sum(getVal(condReStruc,"blockNumTheta"))),
                        thetazi = rr0(sum(getVal(ziReStruc,  "blockNumTheta"))),
                        thetaf  = rr0(numThetaFamily)
@@ -207,7 +227,7 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
             condList, ziList, dispList, condReStruc, ziReStruc,
             family, contrasts, respCol,
             allForm=namedList(combForm,formula,ziformula,dispformula),
-            fr, se, call, verbose, REML, map))
+            fr, se, call, verbose, REML, map, sparseX))
 }
 
 ##' Create X and random effect terms from formula
@@ -217,6 +237,7 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
 ##' @param ranOK random effects allowed here?
 ##' @param type label for model type
 ##' @param contrasts a list of contrasts (see ?glmmTMB)
+##' @param sparse (logical) return sparse model matrix?
 ##' @return a list composed of
 ##' \item{X}{design matrix for fixed effects}
 ##' \item{Z}{design matrix for random effects}
@@ -226,7 +247,7 @@ mkTMBStruc <- function(formula, ziformula, dispformula,
 ##' @importFrom stats model.matrix contrasts
 ##' @importFrom methods new
 ##' @importFrom lme4 findbars nobars
-getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts) {
+getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts, sparse=FALSE) {
     ## fixed-effects model matrix X -
     ## remove random effect parts from formula:
     fixedform <- formula
@@ -255,10 +276,14 @@ getXReTrms <- function(formula, mf, fr, ranOK=TRUE, type="", contrasts) {
         attr(tt, "predvars") <- fix_predvars(pv,tt)
         mf$formula <- tt
         terms_fixed <- terms(eval(mf,envir=environment(fixedform)))
-        ## FIXME: make model matrix sparse?? i.e. Matrix:::sparse.model.matrix()
-        X <- model.matrix(drop.special2(fixedform), fr, contrasts)
+        if (!sparse) {
+            X <- model.matrix(drop.special2(fixedform), fr, contrasts)
+        } else {
+            X <- Matrix::sparse.model.matrix(drop.special2(fixedform), fr, contrasts)
+            ## FIXME? ?sparse.model.matrix recommends MatrixModels::model.Matrix(*,sparse=TRUE)
+            ##  (but we may not need it, and would add another dependency etc.)
+        }
         ## will be 0-column matrix if fixed formula is empty
-
         offset <- rep(0,nobs)
         terms <- list(fixed=terms(terms_fixed))
         if (inForm(fixedform,quote(offset))) {
@@ -469,6 +494,7 @@ binomialType <- function(x) {
 ##' @param REML whether to use REML estimation rather than maximum likelihood.
 ##' @param start starting values, expressed as a list with possible components \code{beta}, \code{betazi}, \code{betad} (fixed-effect parameters for conditional, zero-inflation, dispersion models); \code{b}, \code{bzi} (conditional modes for conditional and zero-inflation models); \code{theta}, \code{thetazi} (random-effect parameters, on the standard deviation/Cholesky scale, for conditional and z-i models); \code{thetaf} (extra family parameters, e.g., shape for Tweedie models).
 ##' @param map a list specifying which parameter values should be fixed to a constant value rather than estimated. \code{map} should be a named list containing factors corresponding to a subset of the internal parameter names (see \code{start} parameter). Distinct factor values are fitted as separate parameter values, \code{NA} values are held fixed: e.g., \code{map=list(beta=factor(c(1,2,3,NA)))} would fit the first three fixed-effect parameters of the conditional model and fix the fourth parameter to its starting value. In general, users will probably want to use \code{start} to specify non-default starting values for fixed parameters. See \code{\link[TMB]{MakeADFun}} for more details.
+##' @param sparseX a named logical vector containing (possibly) elements named "cond", "zi", "disp" to indicate whether fixed-effect model matrices for particular model components should be generated as sparse matrices, e.g. \code{c(cond=TRUE)}. Default is all \code{FALSE}
 ##' @importFrom stats gaussian binomial poisson nlminb as.formula terms model.weights
 ##' @importFrom lme4 subbars findbars mkReTrms nobars
 ##' @importFrom Matrix t
@@ -568,7 +594,8 @@ glmmTMB <- function(
     control=glmmTMBControl(),
     REML=FALSE,
     start=NULL,
-    map=NULL
+    map=NULL,
+    sparseX=NULL
     )
 {
 
@@ -750,7 +777,8 @@ glmmTMB <- function(
                    verbose=verbose,
                    REML=REML,
                    start=start,
-                   map=map)
+                   map=map,
+                   sparseX=sparseX)
 
     ## Allow for adaptive control parameters
     TMBStruc$control <- lapply(control, eval, envir=TMBStruc)
@@ -773,6 +801,7 @@ glmmTMB <- function(
 ##' @param parallel  Numeric; Setting number of OpenMP threads to evaluate
 ##'                  the negative log-likelihood in parallel
 ##' @param optimizer Function to use in model fitting. See \code{Details} for required properties of this function.
+##' @param eigval_check Check eigenvalues of variance-covariance matrix? (This test may be very slow for models with large numbers of fixed-effect parameters.)
 ##' @importFrom TMB openmp
 ##' @details
 ##' The general non-linear optimizer \code{nlminb} is used by
@@ -801,7 +830,9 @@ glmmTMB <- function(
 ##' \itemize{
 ##' \item the first three arguments, in order, are the starting values, objective function, and gradient function;
 ##' \item it also takes a \code{control} argument;
-##' \item it returns a list with elements (at least) \code{convergence} (0 if convergence is successful) and \code{message}
+##' \item it returns a list with elements (at least) \code{par}, \code{objective}, \code{convergence} (0 if convergence is successful) and \code{message}
+##' (the code internally handles output from \code{optim()}, by renaming the \code{value} component to \code{objective})
+##' 
 ##' }
 ##' @examples
 ##' ## fit with default (nlminb) and alternative (optim/BFGS) optimizer
@@ -816,7 +847,8 @@ glmmTMBControl <- function(optCtrl=NULL,
                            optimizer=nlminb,
                            profile=FALSE,
                            collect=FALSE,
-                           parallel = NULL) {
+                           parallel = NULL,
+                           eigval_check = TRUE) {
 
     if (is.null(optCtrl) && identical(optimizer,nlminb)) {
         optCtrl <- list(iter.max=300, eval.max=400)
@@ -834,7 +866,8 @@ glmmTMBControl <- function(optCtrl=NULL,
     ## profile = (length(parameters$beta) >= 2) &&
     ##           (family$family != "tweedie")
     ## (TMB tweedie derivatives currently slow)
-    namedList(optCtrl, profile, collect, parallel, optimizer, optArgs)
+    namedList(optCtrl, profile, collect, parallel, optimizer, optArgs,
+              eigval_check)
 }
 
 ##' collapse duplicated observations
@@ -877,7 +910,23 @@ glmmTMBControl <- function(optCtrl=NULL,
     data.tmb
 }
 
-## FIXME: export fitTMB?
+##' Optimize a TMB model and package results
+##'
+##' This function (called internally by \code{\link{glmmTMB}}) runs
+##' the actual model optimization, after all of the appropriate structures
+##' have been set up. It can be useful to run \code{\link{glmmTMB}} with
+##' \code{doFit=TRUE}, adjust the components as required, and then
+##' finish the fitting process with \code{fitTMB} (however, it is the
+##' user's responsibility to make sure that any modifications 
+##' create an internally consistent final fitted object).
+##' 
+##' @param TMBStruc a list contain
+##' @examples
+##' m0 <- glmmTMB(count ~ mined + (1|site),
+##'              family=poisson, data=Salamanders, doFit=FALSE)
+##' names(m0)
+##' fitTMB(m0)
+##' @export
 fitTMB <- function(TMBStruc) {
 
     control <- TMBStruc$control
@@ -903,7 +952,7 @@ fitTMB <- function(TMBStruc) {
 
     ## avoid repetition; rely on environment for parameters
     optfun <- function() {
-        with(obj,
+        res <- with(obj,
              if( length(par) ) {
                  do.call(control$optimizer,
                          c(list(par, fn, gr,
@@ -912,6 +961,16 @@ fitTMB <- function(TMBStruc) {
              } else {
                  list( par=par, objective=fn(par))
              })
+        ## make optim() results look like nlminb() results (which is
+        ## what glmmTMB is expecting downstream)
+        ## FIXME: what does nloptr output look like?
+        ## nlminb components: par, objective, convergence, message
+        ## optim components: par, value, counts, convergence, message
+        if ("value" %in% names(res)) {
+            res$objective <- res$value
+            res$value <- NULL
+        }
+        return(res)
     }
     
     if (control $ profile) {
@@ -996,14 +1055,23 @@ fitTMB <- function(TMBStruc) {
         warning(paste0("Model convergence problem; ",
                        "non-positive-definite Hessian matrix. ", 
                        "See vignette('troubleshooting')"))
-      } else {
+      } else if (control$eigval_check) {
         eigval <- try(1/eigen(sdr$cov.fixed)$values, silent=TRUE)
+        if (is.complex(eigval)) {
+            ## FIXME: more principled cutoff?
+            if ((maxim <- max(abs(Im(eigval)))) > .Machine$double.eps*10) {
+                eigval <- Re(eigval)
+            } else {
+                stop(sprintf("detected complex eigenvalues of covariance matrix (max(abs(Im))=%g: try se=FALSE?",
+                             maxim))
+            }
+        }
         if( is(eigval, "try-error") || ( min(eigval) < .Machine$double.eps*10 ) ) {
           warning(paste0("Model convergence problem; ",
                        "extreme or very small eigen values detected. ", 
                        "See vignette('troubleshooting')"))
         }
-      }
+     } ## eigenvalue check
     }
 
     if ( !is.null(fit$convergence) && fit$convergence != 0)
@@ -1033,7 +1101,8 @@ fitTMB <- function(TMBStruc) {
                                 reStruc = namedList(condReStruc, ziReStruc),
                                 allForm,
                                 REML,
-                                map))
+                                map,
+                                sparseX))
     ## FIXME: are we including obj and frame or not?
     ##  may want model= argument as in lm() to exclude big stuff from the fit
     ## If we don't include obj we need to get the basic info out
