@@ -15,11 +15,23 @@ namedList <- function (...) {
     setNames(L, nm)
 }
 
-##' @importFrom stats reformulate
 RHSForm <- function(form,as.form=FALSE) {
-    rhsf <- form[[length(form)]]
-    if (as.form) as.formula(substitute(~F,list(F=rhsf)),
-                            env=environment(form)) else rhsf
+    if (!as.form) return(form[[length(form)]])
+    if (length(form)==2) return(form)  ## already RHS-only
+    ## by operating on RHS in situ rather than making a new formula
+    ## object, we avoid messing up existing attributes/environments etc.
+    form[[2]] <- NULL
+    ## assumes response is *first* variable (I think this is safe ...)
+    if (length(vars <- attr(form,"variables"))>0) {
+        attr(form,"variables") <- vars[-2]
+    }
+    if (is.null(attr(form,"response"))) {
+        attr(form,"response") <- 0
+    }
+    if (length(facs <- attr(form,"factors"))>0) {
+        attr(form,"factors") <- facs[-1,]
+    }
+    return(form)
 }
 
 `RHSForm<-` <- function(formula,value) {
@@ -143,8 +155,8 @@ expandAllGrpVar <- function(bb) {
                         return(lapply(expandGrpVar(x[[3]]),
                                       makeOp,x=x[[2]],op=quote(`|`)))
                     } else {
-                        return(makeOp(esfun(x[[2]]),esfun(x[[3]]),
-                                      op=x[[1]]))
+                        return(setNames(makeOp(esfun(x[[2]]),esfun(x[[3]]),
+                                               op=x[[1]]),names(x)))
                     }
                 }
             } ## esfun def.
@@ -173,6 +185,8 @@ head.name <- function(x) { x }
 ##'    it is of the form (xx|gg), then convert it to the default
 ##'    special type; we won't allow pathological cases like
 ##'    ((xx|gg)) ... [can we detect them?]
+##' @examples
+##' splitForm(quote(us(x,n=2)))
 ##' @keywords internal
 fbx <- function(term,debug=FALSE,specials=character(0),
                 default.special="us") {
@@ -227,6 +241,7 @@ fbx <- function(term,debug=FALSE,specials=character(0),
 ##' splitForm(~x+y+(1|(f/g)/h))             ## 'slash'; term
 ##' splitForm(~x+y+(f|g)+cs(1|g)+cs(a|b,stuff))  ## complex special
 ##' splitForm(~(((x+y))))               ## lots of parentheses
+##' splitForm(~1+rr(f|g,n=2))
 ##'
 ##' @author Steve Walker
 ##' @importFrom lme4 nobars
@@ -319,7 +334,10 @@ splitForm <- function(formula,
 ##' noSpecials(y~1+us(1|f))
 ##' noSpecials(y~1+us(1|f),delete=FALSE)
 ##' noSpecials(y~us(1|f))
+##' noSpecials(y~us(1|f), delete=FALSE)
+##' noSpecials(y~us(1|f), debug=TRUE)
 ##' noSpecials(y~us+1)  ## should *not* delete unless head of a function
+##' noSpecials(~us+1)   ## should work on a one-sided formula!
 ##' @export
 ##' @keywords internal
 noSpecials <- function(term, delete=TRUE, debug=FALSE) {
@@ -329,37 +347,42 @@ noSpecials <- function(term, delete=TRUE, debug=FALSE) {
         ##    construct response~1 formula
         as.formula(substitute(R~1,list(R=nospec)),
                    env=environment(term))
-    } else
+    } else {
         nospec
+    }
 }
 
-
-## noSpecials_(y~1+us(1|f))
 noSpecials_ <- function(term,delete=TRUE, debug=FALSE) {
     if (debug) print(term)
     if (!anySpecial(term)) return(term)
     if (length(term)==1) return(term)  ## 'naked' specials
     if (isSpecial(term)) {
         if(delete) {
-            NULL
+            return(NULL)
         } else { ## careful to return  (1|f) and not  1|f:
-            substitute((TERM), list(TERM = term[[2]]))
+            return(substitute((TERM), list(TERM = term[[2]])))
         }
     } else {
-        nb2 <- noSpecials(term[[2]], delete=delete, debug=debug)
+        if (debug) print("not special")
+        nb2 <- noSpecials_(term[[2]], delete=delete, debug=debug)
         nb3 <- if (length(term)==3) {
-                   noSpecials(term[[3]], delete=delete, debug=debug)
+                   noSpecials_(term[[3]], delete=delete, debug=debug)
                } else NULL
-        if (is.null(nb2))
-            nb3
-        else if (is.null(nb3))
-            nb2
-        else {
+        if (is.null(nb2)) {
+            return(nb3)
+        } else if (is.null(nb3)) {
+            if (length(term)==2 && identical(term[[1]], quote(`~`))) { ## special case for one-sided formula
+                term[[2]] <- nb2
+                return(term)
+            } else {
+                return(nb2)
+            }
+        } else {  ## neither term completely disappears
             term[[2]] <- nb2
             term[[3]] <- nb3
-            term
+            return(term)
         }
-    }
+    } 
 }
 
 isSpecial <- function(term) {
@@ -512,6 +535,21 @@ GMRFmarginal <- function(Q, i, ...) {
     ans
 }
 
+# n.b. won't work for terms with more than 2 args ...
+# @examples
+# replaceForm(quote(a(b+x*c(y,z))),quote(y),quote(R))
+# ss <- ~(1 | cask:batch) + (1 | batch)
+# replaceForm(ss,quote(cask:batch),quote(batch:cask))
+replaceForm <- function(term,target,repl) {
+    if (identical(term,target)) return(repl)
+    if (!inForm(term,target)) return(term)
+    if (length(term) == 2) {
+        return(substitute(OP(x),list(OP=term[[1]],x=replaceForm(term[[2]],target,repl))))
+    }
+    return(substitute(OP(x,y),list(OP=term[[1]],
+                                   x=replaceForm(term[[2]],target,repl),
+                                   y=replaceForm(term[[3]],target,repl))))
+}
 
 parallel_default <- function(parallel=c("no","multicore","snow"),ncpus=1) {
     ##  boilerplate parallel-handling stuff, copied from lme4
@@ -525,3 +563,129 @@ parallel_default <- function(parallel=c("no","multicore","snow"),ncpus=1) {
     }
     return(list(parallel=parallel,do_parallel=do_parallel))
 }
+
+##' translate vector of correlation parameters to correlation values
+##' @param theta vector of internal correlation parameters
+##' @return a vector of correlation values
+##' @details This function follows the definition at \url{http://kaskr.github.io/adcomp/classUNSTRUCTURED__CORR__t.html}:
+##' if \eqn{L} is the lower-triangular matrix with 1 on the diagonal and the correlation parameters in the lower triangle, then the correlation matrix is defined as \eqn{\Sigma = D^{-1/2} L L^\top D^{-1/2}}{Sigma = sqrt(D) L L' sqrt(D)}, where \eqn{D = \textrm{diag}(L L^\top)}{D = diag(L L')}. For a single correlation parameter \eqn{\theta_0}{theta0}, this works out to \eqn{\rho = \theta_0/\sqrt{1+\theta_0^2}}{rho = theta0/sqrt(1+theta0^2)}. The function returns the elements of the lower triangle of the correlation matrix, in column-major order.
+##' @examples
+##' th0 <- 0.5
+##' stopifnot(all.equal(get_cor(th0),th0/sqrt(1+th0^2)))
+##' get_cor(c(0.5,0.2,0.5))
+##' @export
+get_cor <- function(theta) {
+    n <- round((1  + sqrt(1+8*length(theta)))/2) ## dim of cor matrix
+    L <- diag(n)
+    L[lower.tri(L)] <- theta
+    cL <- tcrossprod(L)
+    Dh <- diag(1/sqrt(diag(cL)))
+    cc <- Dh %*% cL %*% Dh
+    return(cc[lower.tri(cc)])
+}
+
+match_which <- function(x,y) {
+    which(sapply(y,function(z) x %in% z))
+}
+
+## reassign predvars to have term vars in the right order,
+##  but with 'predvars' values inserted where appropriate
+fix_predvars <- function(pv,tt) {
+    if (length(tt)==3) {
+        ## convert two-sided to one-sided formula
+        tt <- RHSForm(tt, as.form=TRUE)
+    }
+    ## ugh, deparsing again ...
+    tt_vars <- vapply(attr(tt,"variables"),deparse,character(1))[-1]
+    ## remove terminal paren - e.g. match term poly(x, 2) to
+    ##   predvar poly(x, 2, <stuff>)
+    ## beginning of string, including open-paren, colon
+    ##  and *first* comma but not arg ... 
+    init_regexp <- "^([(^:_.[:alnum:]]+).*"
+    tt_vars_short <- gsub(init_regexp,"\\1",tt_vars)
+    if (is.null(pv) || length(tt_vars)==0) return(NULL)
+    new_pv <- quote(list())
+    ## maybe multiple variables per pv term ... [[-1]] ignores head
+    ## FIXME: test for really long predvar strings ????
+    pv_strings <- vapply(pv,deparse,FUN.VALUE=character(1),
+                         width.cutoff=500)[-1]
+    pv_strings <- gsub(init_regexp,"\\1",pv_strings)
+    for (i in seq_along(tt_vars)) {
+        w <- match(tt_vars_short[[i]],pv_strings)
+        if (!is.na(w)) {
+            new_pv[[i+1]] <- pv[[w+1]]
+        } else {
+            ## insert symbol from term vars
+            new_pv[[i+1]] <- as.symbol(tt_vars[[i]])
+        }
+    }
+    return(new_pv)
+}
+
+hasRandom <- function(x) {
+    pl <- getParList(x)
+    return(length(unlist(pl[grep("^theta",names(pl))]))>0)
+}
+
+getParms <- function(parm=NULL, object, full=FALSE) {
+    vv <- vcov(object, full=TRUE)
+    sds <- sqrt(diag(vv))
+    pnames <- names(sds) <- rownames(vv)
+    intnames <- names(object$obj$env$last.par) ## internal names
+    ## "beta" vals may be identified by object$obj$env$random, if REML
+    intnames <- intnames[intnames != "b"]
+    if (length(pnames) != length(sds)) { ## shouldn't happen ...
+        stop("length mismatch between internal and external parameter names")
+    }
+
+    if (is.null(parm)) {
+        if (!full && trivialDisp(object)) {
+            parm <- grep("betad", intnames, invert=TRUE)
+        } else {
+            parm <- seq_along(sds)
+        }
+    }
+    if (is.character(parm)) {
+        if (identical(parm,"theta_")) {
+            parm <- which(intnames=="theta")
+        } else if (identical(parm,"beta_")) {
+            if (trivialDisp(object)) {
+                ## include conditional and zi params
+                ##   but not dispersion params
+                parm <- grep("^beta(zi)?$",intnames)
+            } else {
+                parm <- grep("beta",intnames)
+            }
+        } else if (identical(parm, "disp_") ||
+                   identical(parm, "sigma")) {
+            parm <- grep("^betad", intnames)
+        } else { ## generic parameter vector
+            nparm <- match(parm,pnames)
+            if (any(is.na(nparm))) {
+                stop("unrecognized parameter names: ",
+                     parm[is.na(nparm)])
+            }
+            parm <- nparm
+        }
+    }
+    return(parm)
+}
+
+isREML <- function(x) {
+    if (is.null(REML <- x$modelInfo$REML)) {
+        ## let vcov work with old (pre-REML option) stored objects
+        REML <- FALSE
+    }
+    return(REML)
+}
+
+## action: message, warning, stop
+check_dots <- function(..., action="stop") {
+    L <- list(...)
+    if (length(L)>0) {
+        FUN <- get(action)
+        FUN("unknown arguments: ",
+            paste(names(L), collapse=","))
+    }
+    return(NULL)
+}
